from unittest.mock import patch

from sqldev import create_engine

from ...conftest import get_testing_print_function


def test_tutorial(clear_sqldev):
    from docs_src.tutorial.where import tutorial009 as mod

    mod.sqlite_url = "sqlite://"
    mod.engine = create_engine(mod.sqlite_url)
    calls = []

    new_print = get_testing_print_function(calls)

    with patch("builtins.print", new=new_print):
        mod.main()
    assert calls == [
        [{"name": "Tarantula", "secret_name": "Natalia Roman-on", "age": 32, "id": 4}],
        [{"name": "Black Lion", "secret_name": "Trevor Challa", "age": 35, "id": 5}],
        [
            {
                "name": "Captain North America",
                "secret_name": "Esteban Rogelios",
                "age": 93,
                "id": 7,
            }
        ],
    ]
